From f13e256c5d3486fe56471d274ae5beb816b05bad Mon Sep 17 00:00:00 2001
From: Mason Smith <masonium@gmail.com>
Date: Mon, 17 May 2010 22:09:57 -0700
Subject: [PATCH 1/2] Added type checking to COUNT-PERMUTATIONS, and reimplemnted using %multiply-range
 Fixed SUBFACTORIAL, which produced incorrect answers for N>=12

---
 numbers.lisp |   22 +++++++++++++++-------
 1 files changed, 15 insertions(+), 7 deletions(-)

diff --git a/numbers.lisp b/numbers.lisp
index bf6ef0b..4a40d9e 100644
--- a/numbers.lisp
+++ b/numbers.lisp
@@ -221,18 +221,26 @@ greater then K."
                (%factorial n-k))))))
 
 (defun subfactorial (n)
-  "Subfactorial of the non-negative integer N."
+  "Subfactorial of N, equal to the number of derangements of a set 
+of N objects."
   (check-type n (integer 0))
   (case n
     (0 1)
     (1 0)
     (otherwise
-     (floor (/ (+ 1 (factorial n)) (exp 1))))))
+     (let ((d 0)
+	   (f 1)) ; current factorial
+       (dotimes (k (- n 1))
+	 (setf f (* f (+ k 2)))
+	 (incf d (/ (if (evenp k) 1 -1) f)))
+       (* f d)))))
 
 (defun count-permutations (n &optional (k n))
   "Number of K element permutations for a sequence of N objects.
-R defaults to N"
-  ;; FIXME: Use %multiply-range and take care of 1 and 2, plus
-  ;; check types.
-  (/ (factorial n)
-     (factorial (- n k))))
+K defaults to N."
+  (check-type n (integer 0))
+  (check-type k (integer 0))
+  (assert (>= n k))
+  (if (zerop k)
+      1
+      (%multiply-range (1+ (- n k)) n)))
-- 
1.7.0.4


From 97d169961f71cd47fbc49d673ce0f63da1b15fdd Mon Sep 17 00:00:00 2001
From: Mason Smith <masonium@gmail.com>
Date: Mon, 17 May 2010 22:12:35 -0700
Subject: [PATCH 2/2] Added tests for COUNT-PERMUTATIONS and SUBFACTORIAL

---
 tests.lisp |   27 +++++++++++++++++++++++++++
 1 files changed, 27 insertions(+), 0 deletions(-)

diff --git a/tests.lisp b/tests.lisp
index c139568..aa001bf 100644
--- a/tests.lisp
+++ b/tests.lisp
@@ -1712,3 +1712,30 @@
   (1 2 3)
   nil
   nil)
+
+(deftest count-permutations.1
+    (let ((ns '(0 1 2 3 5 10)))
+      (mapcar #'(lambda (n) (count-permutations n 0)) ns))
+  (1 1 1 1 1 1)
+  (1 1 1 1 1 1))
+
+(deftest count-permutations.2
+    (let ((ns '(1 2 3 5 10)))
+      (mapcar #'(lambda (n) (count-permutations n 1)) ns))
+  (1 2 3 5 10))
+
+(deftest count-permutations.3
+    (let ((ns '(1 2 3 5 10)))
+      (values
+       (mapcar #'(lambda (n) (count-permutations n (- n 1))) ns)
+       (mapcar #'(lambda (n) (count-permutations n n)) ns)))
+  (1 2 6 120 3628800)
+  (1 2 6 120 3628800))
+
+(deftest subfactorial.1
+    (let ((ns '(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21)))
+      (mapcar #'subfactorial ns))
+  (1 0 1 2 9 44 265 1854 14833
+   133496 1334961 14684570 176214841 2290792932
+   32071101049 481066515734 7697064251745 130850092279664
+   2355301661033953 44750731559645106 895014631192902121 18795307255050944540))
-- 
1.7.0.4

